/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 */
package org.teiid.byteman;

import org.teiid.translator.jdbc.TranslatedCommand;
import org.teiid.translator.ExecutionContext;
import org.teiid.language.*;

import org.teiid.core.util.StringUtil;

import java.util.*;
import java.io.*;

import org.jboss.byteman.rule.Rule;  
import org.jboss.byteman.rule.helper.Helper;  


public class TranslatorDeserializeData extends Helper {  
    public static final String SERIALIZE_OUTPUT_DIR_PROPERTY="org.jboss.byteman.serialize.output.dir";
    public static final String OUTPUT_DIR = System.getProperty(SERIALIZE_OUTPUT_DIR_PROPERTY, "./serialize_output");
    
    private static Map command_dirs = new HashMap();
    
    private boolean errorExist = false;
    private File root_dir; 
    
    protected TranslatorDeserializeData(Rule rule) {  
        super(rule);  
        
        root_dir = new File(OUTPUT_DIR);
        if (!root_dir.exists()) {
            root_dir.mkdirs();
        }
    }  
        
    public void initForDeserialize(Object command, Object obj) throws Throwable {
        String command_str = command.toString();
        int cmd_hashcode = command_str.hashCode();

        
        File file_dir = new File(root_dir,  String.valueOf(cmd_hashcode));
        if (!file_dir.exists()) {
            debug("vvvvvvvvvvvvvvvvvvvvvvvvvvv"); 
            debug(String.format( "\nDirectory not found: %s", file_dir.getAbsolutePath()));
            debug("^^^^^^^^^^^^^^^^^^^^^^^^^^^"); 
            errorExist = true;
            return;
        } 
        
        if (file_dir.list() == null || file_dir.list().length == 0) {
            debug("vvvvvvvvvvvvvvvvvvvvvvvvvvv"); 
            debug(String.format( "\nNo files found in directory: %s", file_dir.getAbsolutePath()));
            debug("^^^^^^^^^^^^^^^^^^^^^^^^^^^"); 
            errorExist = true;

            return;
        }  
        
        String command_dir = file_dir.getAbsolutePath();
        command_dirs.put(obj, command_dir);
        debug("vvvvvvvvvvvvvvvvvvvvvvvvvvv"); 
        debug(String.format( "\nDeserializing data for command %s from directory %s", command_str, command_dir));
        debug("^^^^^^^^^^^^^^^^^^^^^^^^^^^"); 

        errorExist = false;
                         
    }
        
        public List deserialize_output(int cnt, Object obj) throws Throwable {
            if (errorExist) return null;
       
            String command_dir = (String) command_dirs.get(obj);
            
            List results = null;
            try{
                File row_data = new File(command_dir, "row_" + String.valueOf(cnt));
                
                debug(String.format( "\nDeserializing file %s", row_data.getAbsolutePath()));
                
                //use buffering
                InputStream file = new FileInputStream( row_data.getAbsolutePath() );
                InputStream buffer = new BufferedInputStream( file );
                ObjectInput input = new ObjectInputStream ( buffer );
                try{
                    //deserialize the List
                    results = (List)input.readObject();
                 }
                finally{
                    input.close();
                }
            }
            catch(ClassNotFoundException ex){
                debug(String.format( "\nCannot perform input. Class not found. %s", ex.getMessage()));
            }
            catch(IOException ex){
                debug(String.format( "\nCannot perform input. %s", ex.getMessage()));
            }
            
            return results;
        }
    
    
}  


