/*
 * JBoss, Home of Professional Open Source.
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 */
package org.teiid.byteman;

import org.teiid.translator.jdbc.TranslatedCommand;
import org.teiid.translator.ExecutionContext;
import org.teiid.language.*;

import org.teiid.core.util.StringUtil;

import java.util.*;
import java.io.*;


import org.jboss.byteman.rule.Rule;  
import org.jboss.byteman.rule.helper.Helper;  


public class TranslatorSerializeData extends Helper {  
    public static final String SERIALIZE_OUTPUT_DIR_PROPERTY="org.jboss.byteman.serialize.output.dir";
    public static final String OUTPUT_DIR = System.getProperty(SERIALIZE_OUTPUT_DIR_PROPERTY, "./serialize_output");
    
    private static Map command_dirs = new HashMap();
    
    private boolean errorExist = false;
    private File root_dir; 
    
    protected TranslatorSerializeData(Rule rule) {  
        super(rule);  
        
        root_dir = new File(OUTPUT_DIR);
        if (!root_dir.exists()) {
            root_dir.mkdirs();
        }
    }  
    
    public void initForSerialize(Object command, Object obj) throws Throwable {
        String command_str = command.toString();
        int cmd_hashcode = command_str.hashCode();
        debug("vvvvvvvvvvvvvvvvvvvvvvvvvvv"); 
        File file_dir = new File(root_dir,  String.valueOf(cmd_hashcode));

       
        if (!file_dir.exists()) {
            file_dir.mkdirs();
        } else {
            file_dir.delete();
            debug(String.format( "\nCleaned out %s directory", file_dir.getAbsolutePath()));
            file_dir.mkdirs();
        }
        
        String command_dir = file_dir.getAbsolutePath();
        command_dirs.put(obj, command_dir);
        debug(String.format( "\nSerializing Output for command %s to directory %s", command_str, command_dir));
        
        write_key(root_dir, cmd_hashcode, command_str);
       
        debug("^^^^^^^^^^^^^^^^^^^^^^^^^^^");                  
    }
    
    public void serialize_output(Object list, int cnt, Object obj) throws Throwable {  
        
        String command_dir = (String) command_dirs.get(obj);
        
         try {
             java.util.ArrayList data = (java.util.ArrayList) list;
             
            File f = new File(command_dir,  "row_" + String.valueOf(cnt));
             
            debug(String.format( "\nSerializing file %s", f.getAbsolutePath()));
             
            OutputStream file = new FileOutputStream( f.getAbsolutePath() );
            OutputStream buffer = new BufferedOutputStream( file );
            ObjectOutput output = new ObjectOutputStream( buffer );
            try {
                 output.writeObject(data);
            } finally {
                output.flush();
                output.close();
            }
            
        } catch(IOException ex){
            debug( "Cannot perform serialize. " + ex);
            return;
        }  
        
    }   
    
    public void write_key(File root_dir, int hashCode, String command) throws Throwable { 
        PrintWriter out = null;
        File f = new File(root_dir, String.valueOf(hashCode) + ".txt");

            
            FileWriter outFile = new FileWriter(f);
            out = new PrintWriter(outFile);
            out.println(command);
            out.close();

        
        debug(String.format( "\nWrote key %s", f.getAbsolutePath()));
            
    }     
    
}  


